
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: rostami@google.com (Afshin Rostamizadeh)

#include <fstream>
#include <iostream>

#include "kernel/lib/io-utilities.h"

namespace kernel {

void AsciiToVector(const string &filename, Vector *vec) {
  string line;
  ifstream file(filename.c_str());
  CHECK(file.is_open());
  while (1) {
    getline(file, line);
    if (file.eof()) break;

    vec->push_back(strtod(line.c_str(), NULL));
  }
  file.close();
}

void AsciiToMatrix(const string &filename, Matrix *mat) {
  string line;
  char *index, *oindex;
  ifstream file(filename.c_str());
  CHECK(file.is_open());
  int i = 0;
  double temp;
  while (1) {
    getline(file, line);
    if (file.eof()) break;

    mat->push_back(Vector());
    (*mat)[i].push_back(strtod(line.c_str(), &index));
    while (*index != '\0' && *index != '\n') {
      oindex = index;
      temp = strtod(oindex, &index);
      if (oindex == index) break;

      (*mat)[i].push_back(temp);
    }
    i++;
  }
  file.close();
}

int SparseAsciiToMatrix(const string &filename, int r, int c, Matrix *mat) {
  string line;
  char *index, *oindex;
  ifstream file(filename.c_str());
  CHECK(file.is_open());
  int i;
  double temp[3];

  for (i = 0; i < r; i++) {
    mat->push_back(Vector(c, 0));
  }

  while (1) {
    getline(file, line);
    if (file.eof()) break;

    temp[0] = strtod(line.c_str(), &index);
    oindex = index;
    temp[1] = strtod(oindex, &index);
    oindex = index;
    temp[2] = strtod(oindex, &index);
    if (oindex == index) return ERR_BADFORMAT;

    if (temp[0] >= r || temp[1] >= c) return ERR_OUTOFBOUNDS;
    (*mat)[static_cast<int>(temp[0])][static_cast<int>(temp[1])] = temp[2];
  }
  file.close();
  return 0;
}

void KernelPrintLibSVM(const Matrix &mat, const Vector &labels) {
  int i,j;
  for (i = 0; i < mat.size(); i++) {
    cout << labels[i] << " 0:" << i+1 << " ";
    for (j = 0; j < mat[i].size(); j++) {
      cout << j+1 << ":" << mat[i][j] << " ";
    }
    cout << endl;
  }
}

} //namespace kernel
